/**
 * DevExtreme (ui/html_editor/modules/toolbar.js)
 * Version: 18.2.0 (build 18275)
 * Build date: Tue Oct 02 2018
 *
 * Copyright (c) 2012 - 2018 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _createClass = function() {
    function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
            var descriptor = props[i];
            descriptor.enumerable = descriptor.enumerable || false;
            descriptor.configurable = true;
            if ("value" in descriptor) {
                descriptor.writable = true
            }
            Object.defineProperty(target, descriptor.key, descriptor)
        }
    }
    return function(Constructor, protoProps, staticProps) {
        if (protoProps) {
            defineProperties(Constructor.prototype, protoProps)
        }
        if (staticProps) {
            defineProperties(Constructor, staticProps)
        }
        return Constructor
    }
}();
var _quill_importer = require("../quill_importer");
var _renderer = require("../../../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);
var _toolbar = require("../../toolbar");
var _toolbar2 = _interopRequireDefault(_toolbar);
var _iterator = require("../../../core/utils/iterator");
var _type = require("../../../core/utils/type");
var _extend = require("../../../core/utils/extend");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _possibleConstructorReturn(self, call) {
    if (!self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return call && ("object" === typeof call || "function" === typeof call) ? call : self
}

function _inherits(subClass, superClass) {
    if ("function" !== typeof superClass && null !== superClass) {
        throw new TypeError("Super expression must either be null or a function, not " + typeof superClass)
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            enumerable: false,
            writable: true,
            configurable: true
        }
    });
    if (superClass) {
        Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass
    }
}
var BaseModule = (0, _quill_importer.getQuill)().import("core/module");
var TOOLBAR_CLASS = "dx-htmleditor-toolbar";
var TOOLBAR_FORMAT_WIDGET_CLASS = "dx-htmleditor-toolbar-format";
var ACTIVE_FORMAT_CLASS = "dx-format-active";
var ToolbarModule = function(_BaseModule) {
    _inherits(ToolbarModule, _BaseModule);

    function ToolbarModule(quill, options) {
        _classCallCheck(this, ToolbarModule);
        var _this = _possibleConstructorReturn(this, (ToolbarModule.__proto__ || Object.getPrototypeOf(ToolbarModule)).call(this, quill, options));
        _this.editorInstance = options.editorInstance;
        _this._formats = {};
        if ((0, _type.isDefined)(options.items)) {
            _this._renderToolbar();
            _this.quill.on("editor-change", function(eventName) {
                _this.updateFormatWidgets()
            })
        }
        return _this
    }
    _createClass(ToolbarModule, [{
        key: "_renderToolbar",
        value: function() {
            var container = this.options.container || this.getContainer();
            var toolbarItems = this.prepareToolbarItems();
            (0, _renderer2.default)(container).addClass(TOOLBAR_CLASS);
            this.toolbarInstance = this.editorInstance._createComponent(container, _toolbar2.default, {
                dataSource: toolbarItems
            })
        }
    }, {
        key: "getContainer",
        value: function() {
            var $container = (0, _renderer2.default)("<div>");
            this.editorInstance.$element().prepend($container);
            return $container
        }
    }, {
        key: "prepareToolbarItems",
        value: function() {
            var _this2 = this;
            var resultItems = [];
            (0, _iterator.each)(this.options.items, function(index, item) {
                var newItem = void 0;
                if ((0, _type.isObject)(item)) {
                    if (item.items && !item.widget) {
                        var selectItemConfig = _this2._prepareSelectItemConfig(item);
                        newItem = _this2._getToolbarItem(selectItemConfig)
                    } else {
                        newItem = _this2._getToolbarItem(item)
                    }
                } else {
                    if ((0, _type.isString)(item)) {
                        var buttonItemConfig = _this2._prepareButtonItemConfig(item);
                        newItem = _this2._getToolbarItem(buttonItemConfig)
                    }
                }
                if (newItem) {
                    resultItems.push(newItem)
                }
            });
            return resultItems
        }
    }, {
        key: "_prepareButtonItemConfig",
        value: function(formatName) {
            var _this3 = this;
            return {
                widget: "dxButton",
                format: formatName,
                options: {
                    text: formatName,
                    onClick: function(e) {
                        if ("clear" === formatName) {
                            _this3.quill.removeFormat(_this3.quill.getSelection())
                        } else {
                            var format = _this3.quill.getFormat(_this3.quill.getSelection());
                            var value = !format[formatName];
                            _this3.quill.format(formatName, value, "user")
                        }
                        _this3.updateFormatWidgets()
                    }
                }
            }
        }
    }, {
        key: "_prepareSelectItemConfig",
        value: function(item) {
            var _this4 = this;
            return {
                widget: "dxSelectBox",
                format: item.format,
                options: {
                    dataSource: item.items,
                    onValueChanged: function(e) {
                        if (!_this4._isReset) {
                            _this4.quill.format(item.format, e.value, "user")
                        }
                    }
                }
            }
        }
    }, {
        key: "_prepareColorItemConfig",
        value: function(format) {
            var _this5 = this;
            return {
                widget: "dxColorBox",
                format: format,
                options: {
                    onValueChanged: function(e) {
                        if (!_this5._isReset) {
                            _this5.quill.format(format, e.value, "user")
                        }
                    }
                }
            }
        }
    }, {
        key: "_getToolbarItem",
        value: function(item) {
            var _this6 = this;
            var baseItem = {
                options: {
                    onInitialized: function(e) {
                        e.component.$element().addClass(TOOLBAR_FORMAT_WIDGET_CLASS);
                        _this6._formats[item.format] = e.component
                    }
                }
            };
            return (0, _extend.extend)(true, {
                location: "before"
            }, item, this.getDefaultConfig(item.format), baseItem)
        }
    }, {
        key: "getDefaultItemsConfig",
        value: function() {
            var _this7 = this;
            return {
                orderedList: {
                    onClick: function() {
                        _this7.quill.format("list", "ordered", "user")
                    }
                },
                bulletList: {
                    onClick: function() {
                        _this7.quill.format("list", "bullet", "user")
                    }
                },
                header: {
                    options: {
                        displayExpr: function(item) {
                            var isHeaderValue = (0, _type.isDefined)(item) && false !== item;
                            return isHeaderValue ? "H" + item : "Normal"
                        }
                    }
                },
                color: this._prepareColorItemConfig("color"),
                background: this._prepareColorItemConfig("background")
            }
        }
    }, {
        key: "getDefaultConfig",
        value: function(format) {
            return this.getDefaultItemsConfig()[format]
        }
    }, {
        key: "updateFormatWidgets",
        value: function() {
            this._resetFormatWidgets();
            var selection = this.quill.getSelection();
            if (!selection) {
                return
            }
            var formats = this.quill.getFormat(selection);
            for (var format in formats) {
                var formatWidget = this._formats[format];
                if (!formatWidget) {
                    continue
                }
                if (void 0 === formatWidget.option("value")) {
                    formatWidget.$element().addClass(ACTIVE_FORMAT_CLASS)
                } else {
                    this._setValueSilent(formatWidget, formats[format])
                }
            }
            if (this._formats.clear && !(0, _type.isEmptyObject)(formats)) {
                this._formats.clear.$element().addClass(ACTIVE_FORMAT_CLASS)
            }
        }
    }, {
        key: "_setValueSilent",
        value: function(widget, value) {
            this._isReset = true;
            widget.option("value", value);
            this._isReset = false
        }
    }, {
        key: "_resetFormatWidgets",
        value: function() {
            var _this8 = this;
            (0, _iterator.each)(this._formats, function(name, widget) {
                widget.$element().removeClass(ACTIVE_FORMAT_CLASS);
                if ("dxSelectBox" === widget.NAME || "dxColorBox" === widget.NAME) {
                    _this8._setValueSilent(widget, null)
                }
            })
        }
    }]);
    return ToolbarModule
}(BaseModule);
module.exports = ToolbarModule;
